/* BLA
 * (C) Copyright Pattern Recognition Group Delft. All rights reserved.
 *
 * MATVECT.C : Matrix routines for N.R. in C matrices and vectors.
 *
 * Made by : W.F. Schmidt
 * SccsId  : %Z%$RCSfile: matvect.c,v $ V%Q% $Revision: 2.5 $ $Date: 1996/10/07 06:50:00 $
 */

#include <stdio.h>
#include <math.h>

#define SPRLIB
#define MATVECT
#include <sprlib.h>

/*
 * MATMULT : Multiply two matrices in NR format and store result in a new
 * matrix. Matrices A[1..m][1..n] and B[1..n][1..s] are input matrices where
 * the dimensions must match and MOut[1..m][1..s] is the resulting matrix.
 * It is assumed that the supplied input (A and B) matrices match with the
 * required format and that the supplied output matrix MOut is also supplied
 * with the proper dimensions.
 *
 * IN MATHEMATICAL FORM : MOut = A * B
 *
 * NOTE : A core dump will probably result if the requirements are violated !
 *
 * Return value : TRUE  if Input matrix == output matrix.
 *                FALSE if no error detected.
 */

int matmult (double **A, int m, int n, int s, double **B, double **MOut)
{
   int i, j, k;

	if ((A == MOut) || (B == MOut))
	{
		sprerror("matmult(): Input matrices must be != output matrix.");
		return TRUE;
	}

   for (i = 1; i <= m; i++)
   {
      for (j = 1; j <= s; j++)
      {
         MOut[i][j] = 0.0;
         for (k = 1; k <= n; k++)
            MOut[i][j] += A[i][k] * B[k][j];
      }
   }
	
	return FALSE;
}

/*
 * MATVECMULT : Multiply a matrix with a vector in NR format and store
 * the result in a new vector. The input matrix must be of the form
 * a[1..m][1..n] and the input vector must be of the form X[1..n]. The
 * supplied output vector VOut[1..m] is assumed to be of the proper dimension.
 *
 * IN MATHEMATICAL FORM : VOut = A * X
 *
 * NOTE : A core dump will probably result if the requirements are violated !
 *
 * Return value : TRUE if an error is detected.
 *                FALSE if no error is detected.
 */

int matvecmult (double **A, int m, int n, double *X, double *VOut)
{
   int i, j;

	if (X == VOut)
	{
		sprerror("matvecmult(): Input vector must be != output vector.");
		return TRUE;
	}

   for (i = 1; i <= m; i++)
   {
      VOut[i] = 0.0;
      for (j = 1; j <= n; j++)
         VOut[i] += A[i][j] * X[j];
   }

	return FALSE;
}

/*
 * VECINMULT : Calculate the innner product ('dot product') of two vectors.
 * The vector must be of the following format X[1..n] and Y[1..n].
 *
 * IN MATHEMATICAL FORM : InProd = X.Y
 *
 * NOTE : A core dump will probably result if the requirements are violated !
 *
 * Return value : The calculated product.
 */

double vecinmult (double *X, int n, double *Y)
{
   int i;
   double InProd;

   InProd = 0.0;
   for (i = 1; i <= n; i++)
      InProd += X[i] * Y[i];

   return InProd;
}

/*
 * MAT_TRANSP : Transpose matrix A to At. This function calculates the
 * transpose of the input matrix A and stores the result in At. The input
 * matrix must have the following form A[1..m][1..n] and the output matrix
 * must have the following form At[1..n][1..m]. If square matrices are
 * supplied A and At may be the same matrix.
 *
 * IN MATHEMATICAL FORM : At = Transpose(A);
 *
 * NOTE : A core dump will probably result if the requirements are violated !
 *
 * Return value : TRUE  If A and At are equal but non square (n != m)
 *                FALSE If no error detected.
 */

int mattransp (double **A, int m, int n, double **At)
{
   int i, j;

   if (A == At)
   {
      double Temp;

      if (n != m)
      {
         sprerror("mattransp(): Only square matrices can be transposed in place.");
         return TRUE;
      }
      for (i = 1; i < n; i++)
      {
         for (j = i + 1; j <= n; j++)
         {
            Temp = A[i][j];
            A[i][j] = A[j][i];
            A[j][i] = Temp;
         }
      }
   }
   else
   {
      for (i = 1; i <= m; i++)
      {
         for (j = 1; j <= n; j++)
            At[j][i] = A[i][j];
      }
   }

   return FALSE;
}

/*
 * MATCOPY : Copy matrix A to B. Matrix A and B must have the following
 * format A[1..m][1..n] B[1..m][1..n].
 *
 * IN MATHEMATICAL FORM : A = B
 *
 * NOTE : A core dump will probably result if the requirements are violated !
 *
 * Return value : none
 */

void matcopy (double **A, int m, int n, double **B)
{
   int i, j;

   for (i = 1; i <= m; i++)
   {
      for (j = 1; j <= n; j++)
         B[i][j] = A[i][j];
   }
}

#undef MATVECT
#undef SPRLIB

/*
 * End of source.
 */
